import desktop
import gtk
import gobject


#
# Class for transparent windows.
#
class GlassWindow(gtk.Window):

    #
    # Constructor.
    #
    def __init__(self, wintype = gtk.WINDOW_TOPLEVEL):

        # transparency level of the window (0..255)
        self.__bg_alpha = 255
        # the background image of the window
        self.__bg_image = None
        # the desktop background
        self.__bg_root = None

        self.__layout = None
        
        # the current window size
        self.__size = (0, 0)

        # remember the current window position to detect movements
        self.__position = (0, 0)


        gtk.Window.__init__(self, wintype)
        self.__layout = gtk.Fixed()
        self.__layout.show()
        gtk.Window.add(self, self.__layout)
        
        self.__bg_root = gtk.Image()
        self.__bg_root.show()
        self.__layout.put(self.__bg_root, 0, 0)
        
        self.__bg_image = gtk.Image()
        self.__bg_image.show()
        self.__layout.put(self.__bg_image, 0, 0)
        
        self.connect("configure-event", self.__on_configure)



    def add(self, child):
    
        self.__layout.put(child, 0, 0)



    #
    # Observer method for the background.
    # Connect this method to the BG watcher.
    #
    def bg_observer(self, src, cmd):

        self.update_bg()
        



    #
    # Sets the background image and the transparency level of the window.
    #
    def set_bg_image(self, filename, alpha):

        self.__bg_alpha = alpha

        # load image
        if (filename):
            # this is an ugly hack for avoiding memory leaks in pygtk
            self.__bg_image.set_from_file(filename)
            self.__bg_image.show()
        else:
            self.__bg_image.hide()                


    #
    # Sets the window size.
    #
    def resize(self, width, height):
    
        self.__size = (width, height)
        gtk.Window.resize(self, width, height)
        


    #
    # Updates the background for transparency.
    #
    def update_bg(self):

        if (not self.window): return
        
        x, y = self.window.get_origin()
        width, height = self.__size
        if (width > 10 and height > 10):
            self.__capture_bg(x, y, width, height)



    #
    # Captures the background to create transparency.
    #
    def __capture_bg(self, x, y, width, height):
                                      
        # try capturing the background
        bg_root = None
        try:
            bg_root = desktop.get_wallpaper(x, y, width, height)
        except:
            try:
                bg_root = desktop.get_wallcolor(width, height)
            except:
                bg_root = gtk.gdk.Pixbuf(0, 1, 8, width, height)
                bg_root.fill(0xddddee)
            #end try
        #end try
        
        if (bg_root): self.__bg_root.set_from_pixbuf(bg_root)




    #
    # Reacts on moving the window.
    #
    def __on_configure(self, src, event):

        pos = self.window.get_origin()
        if (pos != self.__position):
            self.__position = pos
            self.update_bg()
